// myapp.h

#ifndef MYAPP_H_INCLUDED
#define MYAPP_H_INCLUDED

#include "myplain.h"

namespace mylib
{

class CMyApp
{
private:
	static const int MAX_LOAD_STRING = 256;
	static LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

public:
	CMyApp(HINSTANCE hInstance, HINSTANCE hPrevInstance);
	~CMyApp();
	int Run(int nCmdShow);

public:
	HINSTANCE m_hInstance;
	CMyPlain m_doc;

private:
	BOOL ParseCommandLine();

private:
	std::vector<TCHAR> m_DocName;
	TCHAR m_szAppTitle[(MAX_LOAD_STRING+1)];
};
}	// namespace mylib
#endif	// MYAPP_H_INCLUDED
