// myplain.h

#ifndef MYPLAIN_H_INCLUDED
#define MYPLAIN_H_INCLUDED

#include <windows.h>
#include <tchar.h>
#include <vector>

namespace mylib
{
class CMyPlain
{
public:
	CMyPlain();
	CMyPlain(const TCHAR* pszFileName);
	const SIZE& Size(HDC hdc);
	virtual ~CMyPlain();
	virtual int Draw(HDC hdc, int x, int y, PAINTSTRUCT& ps);

private:
	std::vector<std::vector<TCHAR> > m_linebuf;
	int m_nLines;
	int m_nWidth;
	SIZE m_sz;
};

}	// namespace mylib

#endif	// MYPLAIN_H_INCLUDED
