// myappwin.h

#ifndef MYAPPWIN_H_INCLUDED
#define MYAPPWIN_H_INCLUDED

#include "myscroll.h"

namespace mylib
{

class CMyAppWin : public CMyScrollWin
{
public:
	CMyAppWin(LPCTSTR szTitle);

protected:
	virtual BOOL OnDestroy();
	virtual BOOL OnPaint();
};

}	// namespace mylib

#endif	// MYAPPWIN_H_INCLUDED
