#include <windows.h>
#include <tchar.h>
#include "myapp.h"

mylib::CMyApp* g_pApp;

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
	LPSTR lpCmdLine, int nCmdShow)
{
	using namespace mylib;
	// Create my application
	CMyApp app(hInstance, hPrevInstance);
	g_pApp = &app;
	return app.Run(nCmdShow);
}
